/* disk_testkit.c
   --------------

   Disk access functions used under Stewart Smith's Testkit.

   $Id: disk_testkit.c,v 1.7 2003/10/20 07:18:11 stewart Exp $

   (C)2003 Stewart Smith
   Distributed under the GNU Public License

*/

#include <stdio.h>
#include <stdlib.h>
#include "testkit/block_dev.h"
#include "disk.h"

/* disk_new
   --------
   new disk object from testkit block device.
*/
struct fcfs_disk* disk_new(struct block_device *bdev)
{
  struct fcfs_disk* disk;

  disk = (struct fcfs_disk*)malloc(sizeof(struct fcfs_disk));
  disk->os_private = bdev;
  disk->bsize = bdev->block_size;
  disk->blocksnr = bdev->num_blocks;
  disk->sb = NULL;
  return disk;
}

/* disk_free
   ---------
   Close and free the disk.
 */
struct fcfs_disk* disk_free(struct fcfs_disk* disk)
{
  block_dev_close(disk->os_private);
  free(disk);
}

/* Return an empty disk_block for the block. Does not read from disk */
struct fcfs_disk_block* disk_newblock(struct fcfs_disk *disk, sector_t block)
{
  struct fcfs_disk_block* disk_block;

  disk_block = (struct fcfs_disk_block*)malloc(sizeof(struct fcfs_disk_block));
  if(disk_block==NULL)
    {
      fprintf(stderr,"Unable to allocate disk_block\n");
      abort();
    }
  disk_block->os_private = bnew(disk->os_private,block,disk->bsize);
  disk_block->blocknr = block;
  disk_block->bsize = disk->bsize;
  disk_block->data = ((struct buffer_head*)(disk_block->os_private))->b_data;
  ((struct buffer_head*)(disk_block->os_private))->b_count++;
  disk_block->disk = disk;
  
  return disk_block;
}

/* Allocate and return a disk block */
struct fcfs_disk_block* disk_getblock(struct fcfs_disk *disk,sector_t block)
{
  struct fcfs_disk_block* disk_block;
  
  disk_block = (struct fcfs_disk_block*)malloc(sizeof(struct fcfs_disk_block));
  if(disk_block==NULL)
    {
      fprintf(stderr,"Unable to allocate disk_block\n");
      abort();
    }

  //  fprintf(stderr,"DISK_GETBLOCK: 0x%llx\n",block);

  disk_block->os_private = bread(disk->os_private,block,disk->bsize);
  disk_block->blocknr = block;
  disk_block->bsize = disk->bsize;
  disk_block->data = ((struct buffer_head*)(disk_block->os_private))->b_data;
  ((struct buffer_head*)(disk_block->os_private))->b_count++;
  disk_block->disk = disk;

  return disk_block;
}

