/*
  fcfs_newobj.c
  -------------
  
  Program that inserts an object onto an FCFS volume
  from a unix file.

  $Id: fcfs_newobj.c,v 1.5 2003/10/20 07:18:11 stewart Exp $
  
  (C)2003 Stewart Smith
  Distributed under the GPL
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>


#include "testkit/block_dev.h"
#include "testkit/types.h"
#include "testkit/bitops.h"
#include "disk.h"

#include "super_block.h"
#include "onode.h"
#include "onode_index.h"
#include "space_bitmap.h"
#include "fcfs_vfs.h"

#define EXPERIMENTAL

int main(int argc, char* argv[])
{
  struct fcfs_disk* disk;
  struct fcfs_onode_index* index;
  int i;

  if(argc<3)
    {
      fprintf(stderr,"fcfs_newobj\n");
      fprintf(stderr,"-----------\n");
      fprintf(stderr,"$Id: fcfs_newobj.c,v 1.5 2003/10/20 07:18:11 stewart Exp $\n");
      fprintf(stderr,"Usage:\n");
      fprintf(stderr,"\t%s volume file [file2...]\n\n",argv[0]);
      exit(1);
    }

  disk = fcfs_mount(argv[1]);

  index = onode_index_read(disk);
  for(i=2;i<argc;i++)
  {
    struct fcfs_onode1 *node;
    struct fcfs_block_run *onode_br;
    int forknr;
    char data[8200]; int j;
    int infile;
    int rlen;
    struct stat statbuf;

    node = onode1_new(disk);
    onode_br = onode_index_insert(index,node);

    infile = open(argv[i],O_RDONLY);
    fstat(infile,&statbuf);

    /* UNIX source File name fork  */
    onode1_fork_new(disk,onode_br,0x42,strlen(argv[i]),argv[i],1);

    /* Current revision fork */
    forknr = onode1_fork_new(disk,onode_br,0x69,statbuf.st_size,NULL,0);
    onode1_fork_new(disk,onode_br,0x6A,0,NULL,0); /* Revision history fork */
    onode1_fork_new(disk,onode_br,0x6B,0,NULL,0); /* Revision history data fork */

    fprintf(stderr,"%s in fork %d\n",argv[i],forknr);

    j=0;
    while(0 < (rlen = read(infile,data,8192)))
      {	
	//	fprintf(stderr,"*******WRITING %d bytes*******\n",rlen);
	onode1_fork_write(disk,onode_br,forknr,j,(u64)rlen,data);
	j+=rlen;
      }
    close(infile);
    free(node);
    free(onode_br);
  }  
  
  fcfs_umount(disk);

  return 0;
}
