/* onode_index.h
   -------------
   A casually demented B+Tree designed to stay balanced
   and be optimized for access from disk (and in-core cache)
   and not to have to write lots of times.

   Some of the ideas for this came from Reiser4 stuff, except
   I'm allowing unbalanced stuff to make it to disk if needed.
   I'm thinking that if things get really bad, we can run a 
   tree repacker :)

   $Id: onode_index.h,v 1.9 2003/10/20 07:18:11 stewart Exp $

   (C)2003 Stewart Smith
   Distributed under the GNU Public License
*/

#ifndef __ONODE_INDEX_H__
#define __ONODE_INDEX_H__

#include "testkit/types.h"

/* In memory representation of onode_index */
struct fcfs_onode_index {
  //  u64 root_blocknr; /* block run for root of index */
  struct fcfs_onode_index_node *root; /* the node */
  struct fcfs_disk_block *root_block; /* disk block for node */
  struct fcfs_disk *disk;	/* disk the index is on */
};

#define FCFS_ONODE_INDEX_NODE_MAGIC1 0x4f6e4944784e4445ULL  /* OnIDxNDE */

struct fcfs_onode_index_node_item {
  u64 key;
  u64 node_blocknr;
};

struct fcfs_onode_index_node {
  u64 magic1;			/* FCFS_ONODE_INDEX_NODE_MAGIC1 */
  u64 id;			/*Not necessarily unique, used for locking */
  u64 block;
  u64 used;
  struct fcfs_onode_index_node_item items[1];
};

#define FCFS_ONODE_INDEX_LEAF_MAGIC1 0x4f6e4944784c6665ULL  /* OnIDxLfe */

struct fcfs_onode_index_leaf_item {
  u64 key;
  u64 onode_blocknr;
};

struct fcfs_onode_index_leaf {
  u64 magic1;			/* FCFS_ONODE_INDEX_LEAF_MAGIC1 */
  u64 id;			/* Not necessarily unique, used for locking */
  u64 block;
  u64 used;
  struct fcfs_onode_index_leaf_item items[1];
};

u64 onode_index_new_id(struct fcfs_onode_index *index);
struct fcfs_onode_index* onode_index_read(struct fcfs_disk *disk);
struct fcfs_onode_index* onode_index_new(struct fcfs_disk *disk);
int onode_index_write_leaf(struct fcfs_onode_index *index, struct fcfs_onode_index_leaf *leaf);
struct fcfs_disk_block *onode_index_new_node(struct fcfs_onode_index *index);
struct fcfs_onode_index_leaf *onode_index_leaf_new(struct fcfs_onode_index *index, struct fcfs_onode_index_node *parent, int position);
struct fcfs_onode_index_node* onode_index_read_node(struct fcfs_onode_index *index, u64 blocknr);
int onode_index_free_node(struct fcfs_onode_index *index, struct fcfs_onode_index_node* node);
int onode_index_write_node(struct fcfs_onode_index *index, struct fcfs_onode_index_node* node);

int onode_index_new_root(struct fcfs_onode_index *index);
int onode_index_leaf_insert(struct fcfs_disk *disk,struct fcfs_onode_index_leaf *leaf, u64 key, u64 value);
struct fcfs_block_run* onode_index_insert(struct fcfs_onode_index *index, struct fcfs_onode1 *onode);
struct fcfs_disk_block *onode_index_lookup(struct fcfs_onode_index *index,struct fcfs_block_run *onode_br, u64 id);
struct fcfs_onode_index* onode_index_delete(struct fcfs_onode_index* index);

#endif
