/* space_bitmap.c
   --------------
   Simple space allocator using a bitmap

   $Id: space_bitmap.c,v 1.11 2003/10/20 07:18:11 stewart Exp $

   (C)2003 Stewart Smith
   Distributed under the GNU Public License
*/

#include <stdio.h>
#include <stdlib.h>
#include "testkit/block_dev.h"
#include "testkit/bitops.h"
#include "super_block.h"
#include "disk.h"
#include "space_bitmap.h"

#define MAX_DIV(a,b) (((a%b)==0)?(a/b):(a/b)+1)

u64 next_free = 300;

u32 space_bitmap_size(struct fcfs_sb *sb, int ag)
{
  /* if last ag, is remaining no. of blocks */
  if(ag == sb->allocation_groupsnr-1)
    return MAX_DIV(MAX_DIV((sb->ag_blocksnr+(sb->blocksnr%sb->allocation_groupsnr)),8),sb->bsize);
  else
    /* 1 bit per block */
    return MAX_DIV(MAX_DIV(sb->ag_blocksnr,8),sb->bsize);
}

int space_bitmap_allocate_block(struct fcfs_disk *disk, u32 allocation_group,u32 block)
{
  struct fcfs_disk_block *b;

#ifdef DEBUG_SPACE_BITMAP
  printf("ALLOC %u %u\n",allocation_group,block);
#endif

  b = disk_getblock(disk,
		    (allocation_group)*disk->sb->ag_blocksnr
		    +(block/(disk->sb->bsize*8))
		    +1);
  set_bit(block%8,
	  (b->data+((block%(disk->sb->bsize*8))/8)));
  
  disk_writeblock(b);
  disk_freeblock(b);
  return 0;
}

struct fcfs_block_run *ag_allocate_block(struct fcfs_disk *disk, u32 allocation_group, u32 near, u32 blocksnr)
{
  struct fcfs_block_run *br;
  int i;

#ifdef DEBUG_SPACE_BITMAP
  printf("%u %u\n",allocation_group,near);
#endif

  br = (struct fcfs_block_run*)malloc(sizeof(struct fcfs_block_run));
  if(!br)
    {
      fprintf(stderr,"ag_allocate_block: Unable to allocate block_run\n");
      abort();
    }

  br->start = 0;
  br->len = 0;

  if(near+blocksnr > disk->sb->ag_blocksnr)
    blocksnr = disk->sb->ag_blocksnr - near;

  for(i=1;i<=blocksnr+1;i++)
    {
      if(near+i>=disk->sb->ag_blocksnr)
	{allocation_group++;near=0;i=0; br->start=0;br->len=0;/*printf("RESTARTING BLOCK SEARCH IN NEW ALLOCATION GROUP\n");*/}
      if(ag_block_free(disk,allocation_group,near+i))
	{
#ifdef DEBUG_SPACE_BITMAP
	  fprintf(stderr,"BLOCK %lu %lu is free\n",allocation_group,near+i);
#endif
	  if(br->start==0)
	    {  br->start=near+i; br->len=1;}
	  else
	    if((br->start+br->len)==(near+i-1))
	      br->len++;
	}
      else
	{
	  //fprintf(stderr,"BLOCK %lu %lu isn't free\n",allocation_group,near+i);
	  near++;
	  i=0;
	}
    }

  br->allocation_group = allocation_group;

  for(i=0;i<br->len;i++)
    space_bitmap_allocate_block(disk,br->allocation_group,br->start+i);

  next_free = br->start + br->len;

  return br;
}

u64 ag_block_free_last;
struct fcfs_disk_block *ag_block_free_last_block;

int ag_block_free(struct fcfs_disk *disk,u32 allocation_group, u32 block)
{
  struct fcfs_disk_block *b;
  int retval;

#ifdef DEBUG_SPACE_BITMAP
  printf("\t%u %u\n",allocation_group,block);
#endif

  if(block > disk->sb->ag_blocksnr)
    {
      fprintf(stderr,"ERROR: ag_black_free() can't check block that's out of this AG\n");
      abort();
    }

  if(ag_block_free_last==0 ||
     ag_block_free_last!=
     (allocation_group)*disk->sb->ag_blocksnr+(block/(disk->sb->bsize*8))+1
     )
    {
      if(ag_block_free_last!=0) disk_freeblock(ag_block_free_last_block);
      ag_block_free_last = (allocation_group)*disk->sb->ag_blocksnr
	+(block/(disk->sb->bsize*8))
	+1;
      ag_block_free_last_block = b = disk_getblock(disk,ag_block_free_last);
    }
  else
    {
      b = ag_block_free_last_block;
    }
  

  retval =  !test_bit(block%8,
		      (b->data+((block%(disk->sb->bsize*8))/8)));


  return retval;
}
